extern "C" {
#include "Interrupt.h"
#include "BIOS.h"
}

extern u8 _vectors[];
typedef uc16 ui16;
#define NULL 0

class Utils
{
public:
static unsigned int PRNG()
{
    // our initial starting seed is 5323
    static unsigned int nSeed = 5323;
    static unsigned int nX = 0;

    // Take the current seed and generate a new value from it
    // Due to our use of large constants and overflow, it would be
    // very hard for someone to predict what the next number is
    // going to be from the previous one.
    nSeed = (8253729 * nSeed + 2396403);
    nSeed += nX++;

    // Take the seed and return a value between 0 and 32767
    return nSeed & 32767;
}

};

class CSystem
{
public:
  CSystem()
  {
    NVIC_SetVectorTable(NVIC_VectTab_FLASH, (u32)&_vectors);
    __USB_Init();
  }
};

class CDC
{
public:
  static void Clear(ui16 clr)
  {
    __Clear_Screen(clr);
  }
  static void PutPixel(ui16 x, ui16 y, ui16 clr)
  {
    __Point_SCR(x, y);
    __LCD_SetPixl(clr); // len do 32k
  }
  static void Print(ui16 x, ui16 y, ui16 clr, const char* str)
  {
    __Display_Str(x, y, clr, 0, (char*)str);
  }
};

class CApplication : public CSystem
{
public:
  CApplication() : 
    CSystem()
  {
  }

  void operator ()()
  {
    for (int x=0; x<400; x++)
      for (int y=0; y<240; y++)
      {
        u32 dx = (x-200);
        u32 dy = (y-120);
        dx *= dx;
        dy *= dy;
        u32 dl = dx+dy;
        CDC::PutPixel(x, y, (uc16)(dl>>3) & 0x001f);
      }
  }
};

void CPPTest();

class CTestApplication : public CApplication
{
public:
  void operator ()()
  {
    CApplication::operator () ();
    CDC::Print(20, 200, 0xffff, (char*)"Ahoj! DS203 Test");

    CPPTest();

    int x[3] = {20, 380, 220};
    int y[3] = {20, 40, 220};

    int curx = x[0];
    int cury = y[0];

    while(1)
    {
      int c = Utils::PRNG();
      int r = c%3;
      curx = (curx+x[r]) >> 1;
      cury = (cury+y[r]) >> 1;

      __Point_SCR(curx, cury);
      __LCD_SetPixl(c<<1); // len do 32k
    }

  }
};


class CItem 
{
public:
  virtual void Do()
  {
    CDC::Print(0, 0, -1, "CItem");
  };
};

class CItemA : public CItem
{
public:
  virtual void Do()
  {
    CDC::Print(0, 20, -1, "CItemA");
  };
};

class CItemB : public CItem
{
public:
  virtual void Do()
  {
    CDC::Print(0, 40, -1, "CItemB");
  };
};

void CPPTest()
{
  CItem i;
  CItemA ia;
  CItemB ib;

  CItem* arrItems[] = {&i, &ia, &ib};
  for (int i=0; i<3; i++)
    arrItems[i]->Do();
}

int main(void)
{ 
  CTestApplication app;
  app();
  return 0;
}


